package com.xy.service;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.device.EnumAlgorithmPayConfig;
import com.xy.device.EnumDeviceAlgorithmChargingHistoryStatus;
import com.xy.device.EnumDeviceAlgorithmChargingType;
import com.xy.dto.DeviceAlgorithmChargingDto;
import com.xy.entity.DeviceAlgorithmCharging;
import com.xy.entity.DeviceAlgorithmChargingHistory;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceAlgorithmChargingMapper;
import com.xy.utils.DataTime;
import com.xy.utils.Emptys;
import com.xy.utils.R;
import com.xy.utils.SysDictUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 设备算法计费表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-06-29
 */
@Service
@AllArgsConstructor
@Api(tags = "设备算法计费表")
public class DeviceAlgorithmChargingServiceImpl extends ServiceImpl<DeviceAlgorithmChargingMapper, DeviceAlgorithmCharging> implements DeviceAlgorithmChargingService {

    private DeviceAlgorithmChargingHistoryServiceImpl deviceAlgorithmChargingHistoryService;

    @Override
    @ApiOperation("购买")
    public R pay(DeviceAlgorithmChargingDto.Pay pay) {
        JList<DeviceAlgorithmChargingDto.Pay.DeviceAlgorithmCharging> deviceAlgorithmChargings = new JArrayList<>(pay.getDeviceAlgorithmChargings());
        LocalDateTime now = LocalDateTime.now();
        List<DeviceAlgorithmChargingHistory> deviceAlgorithmChargingHistories = new ArrayList<>(deviceAlgorithmChargings.size());
        deviceAlgorithmChargings.forEach(deviceAlgorithmCharging -> {
            DeviceAlgorithmChargingHistory deviceAlgorithmChargingHistory = new DeviceAlgorithmChargingHistory()
                    .setId(YitIdHelper.nextId())
                    .setOrderId(pay.getOrderId())
                    .setDeviceId(deviceAlgorithmCharging.getDeviceId())
                    .setAlgorithmId(deviceAlgorithmCharging.getAlgorithmId())
                    .setChargingSize(deviceAlgorithmCharging.getChargingSize())
                    .setChargingMoney(deviceAlgorithmCharging.getMoney())
                    .setType(deviceAlgorithmCharging.getType())
                    .setChargingType(pay.getPayType())
                    .setCreateTime(now)
                    .setUpdateTime(now);
            deviceAlgorithmChargingHistories.add(deviceAlgorithmChargingHistory);
        });
        //添加设备算法计费历史表
        if (Emptys.check(deviceAlgorithmChargingHistories)) {
            deviceAlgorithmChargingHistoryService.saveBatch(deviceAlgorithmChargingHistories);
        }
        return R.ok();
    }

    @Override
    @ApiOperation("购买回调")
    public R payNotice(DeviceAlgorithmChargingDto.PayNotice payNotice) {
        List<DeviceAlgorithmChargingHistory> list = deviceAlgorithmChargingHistoryService.list(new LambdaQueryWrapper<DeviceAlgorithmChargingHistory>().eq(DeviceAlgorithmChargingHistory::getOrderId, payNotice.getOrderId()));
        if (!Emptys.check(list)) {
            return R.ok();
        }
        JList<DeviceAlgorithmChargingHistory> deviceAlgorithmChargingHistories = new JArrayList<>(list);
        JList<DeviceAlgorithmChargingHistory> infs = new JArrayList<>();
        JList<DeviceAlgorithmChargingHistory> gives = new JArrayList<>();
        LocalDateTime now = LocalDateTime.now();
        //获取算法包月字典
        Map<String, SysDictRedis> algorithmMoonConfigMaps = SysDictUtils.get(EnumAlgorithmPayConfig.Code.CODE.getCode());
        //获取类型字典
        Map<String, SysDictRedis> deviceAlgorithmChargingTypeMaps = SysDictUtils.get(EnumDeviceAlgorithmChargingType.Code.CODE.getCode());
        //处理有效期
        deviceAlgorithmChargingHistories.forEach(deviceAlgorithmChargingHistory -> {
            Integer type = deviceAlgorithmChargingHistory.getType();
            SysDictRedis deviceAlgorithmChargingType2 = deviceAlgorithmChargingTypeMaps.get(EnumDeviceAlgorithmChargingType.N_2.getCode());
            if (type.equals(Integer.valueOf(deviceAlgorithmChargingType2.getValue()))) {
                infs.add(deviceAlgorithmChargingHistory);
                return;
            }
            SysDictRedis deviceAlgorithmChargingType3 = deviceAlgorithmChargingTypeMaps.get(EnumDeviceAlgorithmChargingType.N_3.getCode());
            if (type.equals(Integer.valueOf(deviceAlgorithmChargingType3.getValue()))) {
                gives.add(deviceAlgorithmChargingHistory);
                return;
            }
            SysDictRedis algorithmMoonConfigDict = algorithmMoonConfigMaps.get(deviceAlgorithmChargingHistory.getAlgorithmId().toString());
            DeviceAlgorithmChargingDto.AlgorithmMoonConfig algorithmMoonConfig = JSONUtil.parseObj(algorithmMoonConfigDict.getValue()).toBean(DeviceAlgorithmChargingDto.AlgorithmMoonConfig.class);
            DeviceAlgorithmChargingDto.AlgorithmMoonConfig.Moon moon = algorithmMoonConfig.getMoon();
            Integer chargingSize = deviceAlgorithmChargingHistory.getChargingSize();
            for (Integer integer = 0; integer < chargingSize; integer++) {
                //查询过期时间最大的算法计费
                LambdaQueryWrapper<DeviceAlgorithmCharging> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceAlgorithmCharging>()
                        .eq(DeviceAlgorithmCharging::getDeviceId, deviceAlgorithmChargingHistory.getDeviceId())
                        .eq(DeviceAlgorithmCharging::getAlgorithmId, deviceAlgorithmChargingHistory.getAlgorithmId())
                        .eq(DeviceAlgorithmCharging::getType, type)
                        .orderByDesc(DeviceAlgorithmCharging::getTimeout)
                        .last("limit 1");
                DeviceAlgorithmCharging deviceAlgorithmCharging = getOne(lambdaQueryWrapper);
                int timeout;
                if (deviceAlgorithmCharging == null) {
                    timeout = Integer.valueOf(DataTime.toString(deviceAlgorithmChargingHistory.getCreateTime(), "yyyyMM"));
                } else {
                    LocalDateTime localDateTime = DataTime.toLocal(deviceAlgorithmCharging.getTimeout() + "01000000", "yyyyMMddHHmmss");
                    String s = DataTime.toString(localDateTime, "yyyy-MM-dd HH:mm:ss");
                    String stringAround = DataTime.getStringAround(0, 1, 0, 0, 0, 0, s);
                    timeout = Integer.valueOf(DataTime.toString(DataTime.toLocal(stringAround), "yyyyMM"));
                }
                DeviceAlgorithmCharging deviceAlgorithmChargingInfo = new DeviceAlgorithmCharging()
                        .setDeviceId(deviceAlgorithmChargingHistory.getDeviceId())
                        .setAlgorithmId(deviceAlgorithmChargingHistory.getAlgorithmId())
                        .setUnusedSize(moon.getSize())
                        .setType(type)
                        .setTimeout(timeout)
                        .setCreateTime(now)
                        .setUpdateTime(now);
                save(deviceAlgorithmChargingInfo);
            }
        });
        //处理无期限
        infs.forEach(deviceAlgorithmChargingHistory -> {
            LambdaQueryWrapper<DeviceAlgorithmCharging> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceAlgorithmCharging>()
                    .eq(DeviceAlgorithmCharging::getDeviceId, deviceAlgorithmChargingHistory.getDeviceId())
                    .eq(DeviceAlgorithmCharging::getAlgorithmId, deviceAlgorithmChargingHistory.getAlgorithmId())
                    .eq(DeviceAlgorithmCharging::getType, deviceAlgorithmChargingHistory.getType());
            DeviceAlgorithmCharging deviceAlgorithmCharging = getOne(lambdaQueryWrapper);
            SysDictRedis algorithmMoonConfigDict = algorithmMoonConfigMaps.get(deviceAlgorithmChargingHistory.getAlgorithmId().toString());
            DeviceAlgorithmChargingDto.AlgorithmMoonConfig algorithmMoonConfig = JSONUtil.parseObj(algorithmMoonConfigDict.getValue()).toBean(DeviceAlgorithmChargingDto.AlgorithmMoonConfig.class);
            DeviceAlgorithmChargingDto.AlgorithmMoonConfig.Inf inf = algorithmMoonConfig.getInf();
            if (deviceAlgorithmCharging == null) {
                DeviceAlgorithmCharging deviceAlgorithmChargingInfo = new DeviceAlgorithmCharging()
                        .setDeviceId(deviceAlgorithmChargingHistory.getDeviceId())
                        .setAlgorithmId(deviceAlgorithmChargingHistory.getAlgorithmId())
                        .setUnusedSize(inf.getSize() * deviceAlgorithmChargingHistory.getChargingSize())
                        .setType(deviceAlgorithmChargingHistory.getType())
                        .setTimeout(-1)
                        .setCreateTime(now)
                        .setUpdateTime(now);
                save(deviceAlgorithmChargingInfo);
            } else {
                deviceAlgorithmCharging.setUnusedSize(deviceAlgorithmCharging.getUnusedSize() + (inf.getSize() * deviceAlgorithmChargingHistory.getChargingSize()))
                        .setUpdateTime(now);
                updateById(deviceAlgorithmCharging);
            }
        });
        //处理赠送
        gives.forEach(deviceAlgorithmChargingHistory -> {
            int timeout = Integer.valueOf(DataTime.toString(deviceAlgorithmChargingHistory.getCreateTime(), "yyyyMM"));
            LambdaQueryWrapper<DeviceAlgorithmCharging> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceAlgorithmCharging>()
                    .eq(DeviceAlgorithmCharging::getDeviceId, deviceAlgorithmChargingHistory.getDeviceId())
                    .eq(DeviceAlgorithmCharging::getAlgorithmId, deviceAlgorithmChargingHistory.getAlgorithmId())
                    .eq(DeviceAlgorithmCharging::getType, deviceAlgorithmChargingHistory.getType())
                    .eq(DeviceAlgorithmCharging::getTimeout, timeout);
            DeviceAlgorithmCharging deviceAlgorithmCharging = getOne(lambdaQueryWrapper);
            if (deviceAlgorithmCharging == null) {
                DeviceAlgorithmCharging deviceAlgorithmChargingInfo = new DeviceAlgorithmCharging()
                        .setDeviceId(deviceAlgorithmChargingHistory.getDeviceId())
                        .setAlgorithmId(deviceAlgorithmChargingHistory.getAlgorithmId())
                        .setUnusedSize(deviceAlgorithmChargingHistory.getChargingSize())
                        .setType(deviceAlgorithmChargingHistory.getType())
                        .setTimeout(timeout)
                        .setCreateTime(now)
                        .setUpdateTime(now);
                save(deviceAlgorithmChargingInfo);
            } else {
                deviceAlgorithmCharging.setUnusedSize(deviceAlgorithmCharging.getUnusedSize() + deviceAlgorithmChargingHistory.getChargingSize())
                        .setUpdateTime(now);
                updateById(deviceAlgorithmCharging);
            }
        });
        if (Emptys.check(deviceAlgorithmChargingHistories)) {
            Integer status = SysDictUtils.getValue(EnumDeviceAlgorithmChargingHistoryStatus.Code.CODE.getCode(), EnumDeviceAlgorithmChargingHistoryStatus.N_2.getCode(), Integer.class);
            deviceAlgorithmChargingHistories.forEach(deviceAlgorithmChargingHistory -> deviceAlgorithmChargingHistory.setStatus(status));
            deviceAlgorithmChargingHistoryService.updateBatchById(deviceAlgorithmChargingHistories);
        }
        return R.ok();
    }

}